'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.createHandler = exports.createBuilder = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _lodash = require('lodash');

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const cwd = process.cwd();
const pageTemplatePath = (0, _path.join)(_config.dir, 'page');
const componentTemplatePath = (0, _path.join)(_config.dir, 'component');

const defaultValue = {
	name: 'index',
	indent: 'tab',
	dir: cwd,
	json: false,
	style: 'wxss',
	type: 'page'
};

const formatTemplate = (templateFile, pathname, basename, indent) => {
	let templateStr = (0, _lodash.template)(templateFile)({ basename });
	if (indent !== defaultValue.indent) {
		const intNum = ~~indent;
		const spaces = new Array(intNum).fill(' ').join('');
		templateStr = templateStr.replace(/\t/g, spaces);
	}
	return templateStr;
};

// 如果name只提供页面名称（例如name: 'myPage'），则最终生成的目录，和目前版本的行为一样，是 {project}/pages/myPage/myPage ({project}为app.json所在目录)。
// 如果name是相对路径（例如name: 'path/myPage'），则最终生成的目录是 {project}/pages/path/myPage。
// 如果name是绝对路径（例如name: '/some/path/myPage'），则最终生成的目录是 {project}/some/path/myPage。
// 如果name只填写了路径（例如name: '/some/path/'），则自动把最后的路径当做页面名称，最终生成的目录是{project}/some/path/path。

const parseName = (name, type) => {
	var _parse = (0, _path.parse)(name);

	const dir = _parse.dir,
	      basename = _parse.name,
	      root = _parse.root;

	const defaultRoot = type === 'page' ? 'pages' : 'components';
	const isAbsolutePath = (0, _path.isAbsolute)(name);
	const hasChildPath = !name.split(_path.sep).slice(-1)[0];
	const isPath = !!dir;
	let pathname = name;

	pathname = isAbsolutePath ? name.replace(root, '') : (0, _path.join)(defaultRoot, name);

	if (!hasChildPath && isPath) {
		pathname = pathname.replace(basename, '');
	}

	return { basename, pathname };
};

const getRootDir = dir => (0, _path.resolve)(cwd, dir);

const validateAppJson = rootDir => {
	const exists = _fs2.default.existsSync((0, _path.resolve)(rootDir, 'app.json'));
	if (!exists) {
		throw new Error(`${ rootDir } 目录找不到 app.json 文件！`);
	}
	return true;
};

const addRoute = (rootDir, pathname, basename, indent) => {
	const filename = rootDir + '/app.json';
	const content = _fs2.default.readFileSync(filename, 'utf8');
	const json = JSON.parse(content);
	const newPath = [pathname, basename].filter(Boolean).join('/');
	if (json.pages.indexOf(newPath) < 0) {
		json.pages.push(newPath);
	}
	const formatedIndent = indent === 'tab' ? '\t' : ~~indent;
	const result = JSON.stringify(json, null, formatedIndent);
	_fs2.default.writeFileSync(filename, result);
};

const create = options => {
	if (typeof options !== 'object') {
		throw new Error('参数必须是一个JSON对象。');
	}

	options = Object.assign({}, defaultValue, options);

	var _options = options;
	const dir = _options.dir,
	      name = _options.name,
	      indent = _options.indent,
	      json = _options.json,
	      style = _options.style,
	      type = _options.type;

	const rootDir = getRootDir(dir);

	validateAppJson(rootDir);

	var _parseName = parseName(name, type);

	const pathname = _parseName.pathname,
	      basename = _parseName.basename;


	const fileRoot = (0, _path.resolve)(rootDir, pathname);

	if (!_fs2.default.existsSync(fileRoot)) {
		_mkdirp2.default.sync(fileRoot);
	}

	const filesTypes = ['wxml', 'js', style];
	if (json || type === 'component') {
		filesTypes.push('json');
	}

	filesTypes.forEach(fileType => {
		fileType = fileType.replace(/^\./g, '');

		const filePath = (0, _path.resolve)(fileRoot, `${ basename }.${ fileType }`);

		if (_fs2.default.existsSync(filePath)) {
			throw new Error(`${ filePath } 已存在！`);
		}

		const notStyleFiles = ['js', 'wxml', 'json'];
		const templateExt = notStyleFiles.includes(fileType) ? fileType : 'wxss';
		const templatePath = type === 'page' ? pageTemplatePath : componentTemplatePath;
		const templateFile = _fs2.default.readFileSync(`${ templatePath }/template.${ templateExt }`, 'utf8');
		const content = formatTemplate(templateFile, pathname, basename, indent);
		_fs2.default.writeFileSync(filePath, content);
		console.log(`${ filePath } 创建成功；`);
	});

	if (type === 'page') {
		addRoute(rootDir, pathname, basename, indent);
	}
	console.log('创建结束。');
};

const createPromptItems = options => [{
	message: '请输入小程序源代码根目录 (app.json 所在目录)',
	default: defaultValue.dir,
	type: 'input',
	name: 'dir',
	when: !options.dir,
	validate: answer => {
		const rootDir = getRootDir(answer);
		try {
			return validateAppJson(rootDir);
		} catch (err) {
			return err.message;
		}
	}
}, {
	message: '请选择需要创建的类型',
	default: defaultValue.type,
	type: 'list',
	choices: ['page', 'component'],
	name: 'type',
	when: !options.dir,
	validate: answer => {
		const rootDir = getRootDir(answer);
		try {
			return validateAppJson(rootDir);
		} catch (err) {
			return err.message;
		}
	}
}, {
	message: answers => {
		const type = answers.type;

		const typeText = type === 'page' ? '页面' : '组件';
		return `请输入${ typeText }名称 (可包含路径)`;
	},
	default: defaultValue.name,
	type: 'input',
	name: 'name',
	when: !options.name
}, {
	message: '请输入文件缩进的方式',
	default: defaultValue.indent,
	type: 'input',
	name: 'indent',
	when: !options.indent
}, {
	message: '是否需要生成配置文件 (.json)',
	default: defaultValue.json,
	type: 'confirm',
	name: 'json',
	when: answers => {
		const type = answers.type;

		return type !== 'component' && !options.json;
	}
}, {
	type: 'list',
	message: '请选择样式文件的类型',
	default: defaultValue.style,
	choices: ['wxss', 'scss', 'less', 'pcss', 'styl', '自定义'],
	name: 'style',
	when: !options.style
}, {
	message: '请输入自定义样式文件的类型',
	type: 'input',
	default: defaultValue.style,
	name: 'style',
	when: answers => !options.style && answers.style === '自定义'
}];

const createBuilder = exports.createBuilder = yargs => {
	yargs.usage('\n create-wxapp-page [args] | create-wxapp-page create [args]').options({
		t: {
			alias: 'type',
			desc: '创建类型(仅支持page或component)',
			type: 'string'
		},
		i: {
			alias: 'indent',
			desc: '文件缩进的方式'
		},
		d: {
			alias: 'dir',
			desc: '小程序源代码根目录 (app.json 所在目录)',
			type: 'string'
		},
		n: {
			alias: 'name',
			desc: '生成页面的名称 (可包含路径)',
			type: 'string'
		},
		j: {
			alias: 'json',
			desc: '是否需要生成配置文件 (.json)',
			type: 'boolean'
		},
		s: {
			alias: 'style',
			desc: '样式文件的类型 (如 wcss, scss 等)',
			type: 'string'
		},
		y: {
			alias: 'yes',
			desc: '使用默认值生成相关文件',
			type: 'boolean'
		}
	}).help().argv;
};

const createHandler = exports.createHandler = (() => {
	var _ref = _asyncToGenerator(function* (argv) {
		if (!_fs2.default.existsSync(_config.dir)) {
			(0, _config.init)();
		}

		if (argv.yes) {
			Object.assign({}, defaultValue, argv);
		} else {
			const promptItems = createPromptItems(argv);
			const answers = yield _inquirer2.default.prompt(promptItems);
			Object.keys(answers).forEach(function (key) {
				argv[key] = answers[key];
			});
		}
		create(argv);
	});

	return function createHandler(_x) {
		return _ref.apply(this, arguments);
	};
})();